function y = tucalc(x)
% IPTUBE calculates vacuum tube plate current in Amperes.
% The following parameters have changed from the 1996 model:
% KGX = KG1^EX is used inside exponent for optimization.
% KVB for TRIODE (now has units of volts).
% 8/25/2001: I'm returning them to the original model for compatibility.

global TubeType MU EX KG1 KGX KP KVB Vp Vg Vs Vsi Idata ERR VCT PENFN NOPT SOLVER

kvbb = KVB;

exx = EX;
if (strcmp(TubeType,'TRIODE') | (max(Vsi)-min(Vsi))/max(Vsi)>.3)
   exx = EX*x(2);  % Optimize on EX.
end

if (strcmp(TubeType,'TRIODE'))		% TRIODE
   if (length(x)>=5 & (max(Vg)+VCT)>eps)  kvbb = KVB*x(5)^2;  end
   E1 = Vp/(KP*x(4)).*log(1+exp(KP*x(4).* ...
      (1/(MU*x(1))+(Vg+VCT)./sqrt(kvbb+Vp.^2))));
   y = (E1./(KGX*x(3))).^exx.*(1+sign(E1));
elseif (strcmp(TubeType,'PENTODE'))		% PENTODE
   if (length(x)>=5)  kvbb = KVB*x(5);  end
   E1 = Vs./(KP*x(4)).*log(1+exp(KP*x(4).*(1/(MU*x(1))+(Vg+VCT)./Vs)));
   y = (E1./(KGX*x(3))).^exx.*(1+sign(E1));  % .*atan(Vp./kvbb);
   if     (PENFN == 2) y = pi/2*y.*(1-exp(-2*Vp/(pi*kvbb)));		% intermediate.
   elseif (PENFN == 3) y = pi/2*y.*tanh(2*Vp/(pi*kvbb));		% sharpest.
   else                y = y.*atan(Vp./kvbb);	% PENFN==1: roundest. In article.
   end
else error(['Bad tube type: ',TubeType]);
end
